@@ -128,38 +128,6 @@ PyTypeObject PyArMember_Type = {
     armember_getset,                     // tp_getset
 };
 
-
-static const char *filefd_doc=
-   "Internal helper type, representing a FileFd.";
-PyTypeObject PyFileFd_Type = {
-    PyVarObject_HEAD_INIT(&PyType_Type, 0)
-    "apt_inst.__FileFd"        ,                 // tp_name
-    sizeof(CppPyObject<FileFd>),         // tp_basicsize
-    0,                                        // tp_itemsize
-    // Methods
-    CppDealloc<FileFd>,                  // tp_dealloc
-    0,                                   // tp_print
-    0,                                   // tp_getattr
-    0,                                   // tp_setattr
-    0,                                   // tp_compare
-    0,                                   // tp_repr
-    0,                                   // tp_as_number
-    0,                                   // tp_as_sequence
-    0,                                   // tp_as_mapping
-    0,                                   // tp_hash
-    0,                                   // tp_call
-    0,                                   // tp_str
-    0,                                   // tp_getattro
-    0,                                   // tp_setattro
-    0,                                   // tp_as_buffer
-    Py_TPFLAGS_DEFAULT |                 // tp_flags
-    Py_TPFLAGS_HAVE_GC,
-    filefd_doc,                          // tp_doc
-    CppTraverse<FileFd>,                 // tp_traverse
-    CppClear<FileFd>,                    // tp_clear
-};
-
-
 // We just add an inline method and should thus be ABI compatible in a way that
 // we can simply cast ARArchive instances to PyARArchiveHack.
 class PyARArchiveHack : public ARArchive
@@ -171,7 +139,7 @@ public:
 };
 
 struct PyArArchiveObject : public CppPyObject<PyARArchiveHack*> {
-    CppPyObject<FileFd> *Fd;
+    FileFd Fd;
 };
 
 static const char *ararchive_getmember_doc =
@@ -217,7 +185,7 @@ static PyObject *ararchive_extractdata(PyArArchiveObject *self, PyObject *args)
                      "Member '%s' is too large to read into memory",name.path);
         return 0;
     }
-    if (!self->Fd->Object.Seek(member->Start))
+    if (!self->Fd.Seek(member->Start))
         return HandleErrors();
 
     char* value;
@@ -228,7 +196,7 @@ static PyObject *ararchive_extractdata(PyArArchiveObject *self, PyObject *args)
                      "Member '%s' is too large to read into memory",name.path);
         return 0;
     }
-    self->Fd->Object.Read(value, member->Size, true);
+    self->Fd.Read(value, member->Size, true);
     PyObject *result = PyBytes_FromStringAndSize(value, member->Size);
     delete[] value;
     return result;
@@ -306,7 +274,7 @@ static PyObject *ararchive_extract(PyArArchiveObject *self, PyObject *args)
         PyErr_Format(PyExc_LookupError,"No member named '%s'",name.path);
         return 0;
     }
-    return _extract(self->Fd->Object, member, target);
+    return _extract(self->Fd, member, target);
 }
 
 static const char *ararchive_extractall_doc =
@@ -325,7 +293,7 @@ static PyObject *ararchive_extractall(PyArArchiveObject *self, PyObject *args)
     const ARArchive::Member *member = self->Object->Members();
 
     do {
-        if (_extract(self->Fd->Object, member, target) == 0)
+        if (_extract(self->Fd, member, target) == 0)
             return 0;
     } while ((member = member->Next));
     Py_RETURN_TRUE;
@@ -352,10 +320,10 @@ static PyObject *ararchive_gettar(PyArArchiveObject *self, PyObject *args)
         return 0;
     }
 
-    PyTarFileObject *tarfile = (PyTarFileObject*)CppPyObject_NEW<ExtractTar*>(self->Fd,&PyTarFile_Type);
-    new (&tarfile->Fd) FileFd(self->Fd->Object.Fd());
+    PyTarFileObject *tarfile = (PyTarFileObject*)CppPyObject_NEW<ExtractTar*>(self,&PyTarFile_Type);
+    new (&tarfile->Fd) FileFd(self->Fd.Fd());
     tarfile->min = member->Start;
-    tarfile->Object = new ExtractTar(self->Fd->Object, member->Size, comp);
+    tarfile->Object = new ExtractTar(self->Fd, member->Size, comp);
     return HandleErrors(tarfile);
 }
 
@@ -422,52 +390,36 @@ static PyObject *ararchive_new(PyTypeObject *type, PyObject *args,
                                PyObject *kwds)
 {
     PyObject *file;
+    PyArArchiveObject *self;
     PyApt_Filename filename;
     int fileno;
     if (PyArg_ParseTuple(args,"O:__new__",&file) == 0)
         return 0;
 
-    PyApt_UniqueObject<PyArArchiveObject> self(NULL);
     // We receive a filename.
     if (filename.init(file)) {
-        self.reset((PyArArchiveObject*) CppPyObject_NEW<ARArchive*>(0,type));
-        self->Fd = CppPyObject_NEW<FileFd>(NULL, &PyFileFd_Type);
-        new (&self->Fd->Object) FileFd(filename,FileFd::ReadOnly);
+        self = (PyArArchiveObject *)CppPyObject_NEW<ARArchive*>(0,type);
+        new (&self->Fd) FileFd(filename,FileFd::ReadOnly);
     }
     // We receive a file object.
     else if ((fileno = PyObject_AsFileDescriptor(file)) != -1) {
         // Clear the error set by PyObject_AsString().
         PyErr_Clear();
-        self.reset((PyArArchiveObject*) CppPyObject_NEW<ARArchive*>(NULL,type));
-        self->Fd = CppPyObject_NEW<FileFd>(file, &PyFileFd_Type);
-        new (&self->Fd->Object) FileFd(fileno,false);
+        self = (PyArArchiveObject *)CppPyObject_NEW<ARArchive*>(file,type);
+        new (&self->Fd) FileFd(fileno,false);
     }
     else {
         return 0;
     }
-    self->Object = (PyARArchiveHack*)new ARArchive(self->Fd->Object);
+    self->Object = (PyARArchiveHack*)new ARArchive(self->Fd);
     if (_error->PendingError() == true)
         return HandleErrors();
-    return self.release();
-}
-
-static int ararchive_traverse(PyObject *_self, visitproc visit, void* arg)
-{
-    PyArArchiveObject *self = (PyArArchiveObject*)_self;
-    Py_VISIT(self->Fd);
-    return CppTraverse<ARArchive*>(self, visit, arg);
-}
-
-static int ararchive_clear(PyObject *_self)
-{
-    PyArArchiveObject *self = (PyArArchiveObject*)_self;
-    Py_CLEAR(self->Fd);
-    return CppClear<ARArchive*>(self);
+    return self;
 }
 
 static void ararchive_dealloc(PyObject *self)
 {
-    ararchive_clear(self);
+    ((PyArArchiveObject *)(self))->Fd.~FileFd();
     CppDeallocPtr<ARArchive*>(self);
 }
 
@@ -521,8 +473,8 @@ PyTypeObject PyArArchive_Type = {
     Py_TPFLAGS_DEFAULT |                 // tp_flags
     Py_TPFLAGS_HAVE_GC,
     ararchive_doc,                       // tp_doc
-    ararchive_traverse,                  // tp_traverse
-    ararchive_clear,                     // tp_clear
+    CppTraverse<ARArchive*>,        // tp_traverse
+    CppClear<ARArchive*>,           // tp_clear
     0,                                   // tp_richcompare
     0,                                   // tp_weaklistoffset
     (getiterfunc)ararchive_iter,         // tp_iter
@@ -577,10 +529,10 @@ static PyObject *_gettar(PyDebFileObject *self, const ARArchive::Member *m,
 {
     if (!m)
         return 0;
-    PyTarFileObject *tarfile = (PyTarFileObject*)CppPyObject_NEW<ExtractTar*>(self->Fd,&PyTarFile_Type);
-    new (&tarfile->Fd) FileFd(self->Fd->Object.Fd());
+    PyTarFileObject *tarfile = (PyTarFileObject*)CppPyObject_NEW<ExtractTar*>(self,&PyTarFile_Type);
+    new (&tarfile->Fd) FileFd(self->Fd.Fd());
     tarfile->min = m->Start;
-    tarfile->Object = new ExtractTar(self->Fd->Object, m->Size, comp);
+    tarfile->Object = new ExtractTar(self->Fd, m->Size, comp);
     return tarfile;
 }
 
@@ -601,7 +553,7 @@ static PyObject *debfile_get_tar(PyDebFileObject *self, const char *Name)
         Member = AR.FindMember(std::string(Name).append(c->Extension).c_str());
         if (Member == NULL)
             continue;
-        Compressor = c->Name;
+        Compressor = c->Binary;
         break;
     }
 
@@ -627,16 +579,16 @@ static PyObject *debfile_get_tar(PyDebFileObject *self, const char *Name)
 
 static PyObject *debfile_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
-    PyApt_UniqueObject<PyDebFileObject> self((PyDebFileObject*)ararchive_new(type, args, kwds));
+    PyDebFileObject *self = (PyDebFileObject*)ararchive_new(type, args, kwds);
     if (self == NULL)
         return NULL;
 
     // DebFile
-    self->control = debfile_get_tar(self.get(), "control.tar");
+    self->control = debfile_get_tar(self, "control.tar");
     if (self->control == NULL)
         return NULL;
 
-    self->data = debfile_get_tar(self.get(), "data.tar");
+    self->data = debfile_get_tar(self, "data.tar");
     if (self->data == NULL)
         return NULL;
 
@@ -645,14 +597,14 @@ static PyObject *debfile_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
         return PyErr_Format(PyAptError, "No debian archive, missing %s",
                             "debian-binary");
 
-    if (!self->Fd->Object.Seek(member->Start))
+    if (!self->Fd.Seek(member->Start))
         return HandleErrors();
 
     char* value = new char[member->Size];
-    self->Fd->Object.Read(value, member->Size, true);
+    self->Fd.Read(value, member->Size, true);
     self->debian_binary = PyBytes_FromStringAndSize(value, member->Size);
     delete[] value;
-    return self.release();
+    return self;
 }
 
 static int debfile_traverse(PyObject *_self, visitproc visit, void* arg)
