@@ -36,6 +36,10 @@
  * Exception class for almost all Python errors
  */
 extern PyObject *PyAptError;
+/**
+ * Exception class for invalidated cache objects.
+ */
+extern PyObject *PyAptCacheMismatchError;
 
 #if PYTHON_API_VERSION < 1013
 typedef int Py_ssize_t;
